@@ -1,59 +1,57 @@
-## TO DO 
+## TO DO
 ##
 ##   - 'best' and 'exact' options for nsamp, as in CovMcd
 ##   - start reimplementing parts of the S-FAST algorithm in C
 ##
-CovSest <- function(x, 
+CovSest <- function(x,
                     bdp=0.5,
                     arp=0.1,
-                    eps=1e-5, 
-                    maxiter=120, 
-                    nsamp=500, 
-                    seed=NULL, 
+                    eps=1e-5,
+                    maxiter=120,
+                    nsamp=500,
+                    seed=NULL,
                     trace=FALSE,
-                    tolSolve=10e-14,
-                    method=c("sfast", "surreal", "bisquare", "rocke"),
+                    tolSolve=1e-14,
+                    method=c("sfast", "surreal", "bisquare", "rocke", "suser"),
                     control,
-                    t0, 
-                    S0, 
+                    t0,
+                    S0,
                     initcontrol
                 )
 {
     ## Compute the constant kp (used in Tbsc)
     Tbsb <- function(c, p)
     {
-        ksiint <- function(c, s, p) 
+        ksiint <- function(c, s, p)
         {
-            (2^s)*gamma(s + p/2)*pgamma(c^2/2, s + p/2)/gamma(p/2)
+            (2^s) * gamma(s + p/2) * pgamma(c^2/2, s + p/2)/gamma(p/2)
         }
 
-        y1 = ksiint(c,1,p)*3/c-ksiint(c,2,p)*3/(c^3)+ksiint(c,3,p)/(c^5);
-        y2 = c*(1-pchisq(c^2,p));
-        return(y1+y2)
+        y1 = ksiint(c,1,p)*3/c - ksiint(c,2,p)*3/(c^3) + ksiint(c,3,p)/(c^5)
+        y2 = c*(1-pchisq(c^2,p))
+        return(y1 + y2)
     }
 
-    ## Compute the tunning constant c1 for the  S-estimator with 
+    ## Compute the tunning constant c1 for the  S-estimator with
     ##  Tukey's biweight function given the breakdown point (bdp)
-    ##  and the dimension p    
+    ##  and the dimension p
     Tbsc <- function(bdp, p)
     {
-        cnew = sqrt(qchisq(1 - bdp, p))
         eps = 1e-8
         maxit = 1e3
-        iter = 1
-        diff = 1e6
+        cnew <- cold <- sqrt(qchisq(1 - bdp, p))
 
         ## iterate until the change is less than the tollerance or
         ##  max number of iterations reached
-        while(diff > eps  && iter < maxit)
+        for(iter in 1:maxit)
         {
-            cold <- cnew
             cnew <- Tbsb(cold, p)/bdp
-            diff <- abs(cold - cnew)
-            iter <- iter + 1
+            if(abs(cold - cnew) <= eps)
+                break
+            cold <- cnew
         }
         return(cnew)
-    } 
+    }
 
     ## Analize and validate the input parameters ...
     ## if a control object was supplied, take the option parameters from it,
@@ -67,7 +65,7 @@ CovSest <- function(x,
         if(arp == defcontrol@arp)           arp <- control@arp          # for rocke type
         if(eps == defcontrol@eps)           eps <- control@eps          # for bisquare and rocke
         if(maxiter == defcontrol@maxiter)   maxiter <- control@maxiter  # for bisquare and rocke
-        
+
         if(nsamp == defcontrol@nsamp)       nsamp <- control@nsamp
         if(is.null(seed) || seed == defcontrol@seed)         seed <- control@seed
         if(trace == defcontrol@trace)       trace <- control@trace
@@ -90,26 +88,38 @@ CovSest <- function(x,
                     dimnames = list(names(x), deparse(substitute(x))))
 
     xcall <- match.call()
-    n <- nrow(x)
-    p <- ncol(x)
-    
-    if(method == "surreal" && nsamp == 500)   # default
+
+    ## drop all rows with missing values (!!) :
+    na.x <- !is.finite(x %*% rep(1, ncol(x)))
+    ok <- !na.x
+    x <- x[ok, , drop = FALSE]
+    dx <- dim(x)
+    if(!length(dx))
+        stop("All observations have missing values!")
+    dimn <- dimnames(x)
+    n <- dx[1]
+    p <- dx[2]
+
+    if(method == "surreal" && nsamp == 500)     # default
         nsamp = 600*p
 
+    if(method == "sfast" && nsamp == 500)       # default. For the 'fast' method set the default number of samples to 20
+        nsamp = 20
+
     ## compute the constants c1 and kp
     c1 = Tbsc(bdp, p)
     kp = (c1/6) * Tbsb(c1, p)
-    
+
     if(trace)
-        cat("\nFAST-S : bdp, p, c1, kp=", bdp, p, c1, kp, "\n")
+        cat("\nFAST-S...: bdp, p, c1, kp=", bdp, p, c1, kp, "\n")
 
-    
-    mm <- if(method == "sfast")         ..fastSloc(x, nsamp=nsamp, kp=kp, cc=c1, trace=trace)
+    mm <- if(method == "sfast")         ..CSloc(x, nsamp=nsamp, kp=kp, cc=c1, trace=trace)
+          else if(method == "suser")    ..fastSloc(x, nsamp=nsamp, kp=kp, cc=c1, trace=trace)
           else if(method == "surreal")  ..covSURREAL(x, nsamp=nsamp, kp=kp, c1=c1, trace=trace, tol.inv=tolSolve)
-          else if(method == "bisquare") ..covSBic(x, arp, eps, maxiter, t0, S0, nsamp, seed, initcontrol)
-          else                          ..covSRocke(x, arp, eps, maxiter, t0, S0, nsamp, seed, initcontrol)
+          else if(method == "bisquare") ..covSBic(x, arp, eps, maxiter, t0, S0, nsamp, seed, initcontrol, trace=trace)
+          else                          ..covSRocke(x, arp, eps, maxiter, t0, S0, nsamp, seed, initcontrol, trace=trace)
 
-    ans <- new("CovSest", 
+    ans <- new("CovSest",
                call = xcall,
                iter=mm$iter,
                crit=mm$crit,
@@ -122,20 +132,18 @@ CovSest <- function(x,
 }
 
 ##
-##  A fast procedure to compute an S-estimator similar to the one proposed 
+##  A fast procedure to compute an S-estimator similar to the one proposed
 ##  for regression by Salibian-Barrera, M. and Yohai, V.J. (2005),
-##  "A fast algorithm for S-regression estimates". This version for 
-##  multivariate location/scatter is adapted from the one implemented 
-##  by Kristel Joossens, K.U. Leuven, Belgium 
-##  and Ella Roelant, Ghent University, Belgium.
+##  "A fast algorithm for S-regression estimates". This current C implemention
+##  is by Valentin Todorov.
 ##
 ## Input:
 ##      x      - a data matrix of size (n,p)
 ##      nsamp  - number of sub-samples (default=20)
 ##      k      - number of refining iterations in each subsample (default=2)
-##      best.r - number of "best betas" to remember from the subsamples. 
+##      best.r - number of "best betas" to remember from the subsamples.
 ##                  These will be later iterated until convergence (default=5)
-##      kp, cc  - tunning constants for the  S-estimator with Tukey's biweight 
+##      kp, cc  - tunning constants for the  S-estimator with Tukey's biweight
 ##                function given the breakdown point (bdp) and the dimension p
 ##
 ## Output: a list with components
@@ -143,13 +151,87 @@ CovSest <- function(x,
 ##      cov     - robust estimate of scatter (matrix: p,p)
 ##      crit    - value of the objective function (number)
 ##
-..fastSloc <- function(x, nsamp, k=2, best.r=5, kp, cc, trace=FALSE) 
+..CSloc <- function(x, nsamp=20, kstep=2, best.r=5, kp, cc, trace=FALSE)
+{
+    dimn <- dimnames(x)
+    if(!is.matrix(x))
+        x = as.matrix(x)
+
+    n <- nrow(x)
+    p <- ncol(x)
+
+    maxit <- 200    # max number of iterations
+    rtol <- 1e-7    # convergence tolerance
+
+    b <- .C("sest",
+	    as.double(x),
+	    as.integer(n),
+	    as.integer(p),
+	    as.integer(nsamp),
+        center = as.double(rep(0,p)),
+        cov = as.double(rep(0,p*p)),
+        scale = as.double(0),
+	    as.double(cc),
+	    as.double(kp),
+	    as.integer(best.r),                    # number of (refined) to be retained for full iteration
+	    as.integer(kstep),                     # number of refining steps for each candidate
+	    as.integer(maxit),                     # number of (refined) to be retained for full iteration
+	    as.double(rtol),
+	    converged = logical(1)
+	    )
+
+    scale <- b$scale
+    center <- b$center
+    cov <- matrix(b$cov, nrow=p)
+
+    if(scale < 0)
+	   stop("C function sest() exited prematurely")
+
+    ## names(b)[names(b) == "k.max"] <- "k.iter" # maximal #{refinement iter.}
+    ## FIXME: get 'res'iduals from C
+
+    if(!is.null(nms <- dimn[[2]])) {
+        names(center) <- nms
+        dimnames(cov) <- list(nms,nms)
+    }
+
+    return(list(
+               center=as.vector(center),
+               cov=cov,
+               crit=scale,
+               iter=nsamp,
+               method="S-estimates: S-FAST"))
+}
+
+##
+##  A fast procedure to compute an S-estimator similar to the one proposed
+##  for regression by Salibian-Barrera, M. and Yohai, V.J. (2005),
+##  "A fast algorithm for S-regression estimates". This version for
+##  multivariate location/scatter is adapted from the one implemented
+##  by Kristel Joossens, K.U. Leuven, Belgium
+##  and Ella Roelant, Ghent University, Belgium.
+##
+## Input:
+##      x      - a data matrix of size (n,p)
+##      nsamp  - number of sub-samples (default=20)
+##      k      - number of refining iterations in each subsample (default=2)
+##      best.r - number of "best betas" to remember from the subsamples.
+##                  These will be later iterated until convergence (default=5)
+##      kp, cc  - tunning constants for the  S-estimator with Tukey's biweight
+##                function given the breakdown point (bdp) and the dimension p
+##
+## Output: a list with components
+##      center  - robust estimate of location (vector: length)
+##      cov     - robust estimate of scatter (matrix: p,p)
+##      crit    - value of the objective function (number)
+##
+..fastSloc <- function(x, nsamp, k=2, best.r=5, kp, cc, trace=FALSE)
 {
     ## NOTES:
     ##  - in the functions rho, psi, and scaledpsi=psi/u (i.e. the weight function)
-    ##      is used |x| <= c1 
+    ##      is used |x| <= c1
     ##
-    ##  - function resdis() to compute the distances is used instead of 
+    ##  - function resdis() to compute the distances is used instead of
     ##      mahalanobis() - slightly faster
 
     ##  The bisquare rho function:
@@ -158,7 +240,7 @@ CovSest <- function(x,
     ##  rho(x) =    |
     ##              |   c1^2/6                                  |x| > c1
     ##
-    rho <- function(u, cc) 
+    rho <- function(u, cc)
     {
         w <- abs(u) <= cc
         v <- (u^2/2 * (1 - u^2/cc^2 + u^4/(3*cc^4))) * w + (1-w) * (cc^2/6)
@@ -178,9 +260,9 @@ CovSest <- function(x,
         pp <- u - 2 * u^3/c1^2 + u^5/c1^4
         pp*(abs(u) <= c1)
     }
-    
+
     ## weight function = psi(u)/u
-    scaledpsi <- function(u, cc) 
+    scaledpsi <- function(u, cc)
     {
         ##ifelse(abs(xx) < c1, xx - 2 * xx^3/c1^2 + xx^5/c1^4, 0)
         pp <- (1 - (u/cc)^2)^2
@@ -189,10 +271,10 @@ CovSest <- function(x,
     }
 
     ## the objective function, we solve loss.S(u, s, cc) = b for "s"
-    loss.S <- function(u, s, cc) 
+    loss.S <- function(u, s, cc)
         mean(rho(u/s, cc))
 
-    norm <- function(x) 
+    norm <- function(x)
         sqrt(sum(x^2))
 
     ## Returns square root of the mahalanobis distances of x with respect to mu and sigma
@@ -206,28 +288,27 @@ CovSest <- function(x,
     }
 
     ##  Computes Tukey's biweight objective function (scale)
-    ##  (respective to the mahalanobis distances u) using the 
+    ##  (respective to the mahalanobis distances u) using the
     ##  rho() function and the konstants kp and c1
-    scaleS <- function(u, kp, c1, initial.sc=median(abs(u))/.6745) 
+    scaleS <- function(u, kp, c1, initial.sc=median(abs(u))/.6745)
     {
         ## find the scale, full iterations
-        max.it <- 200
-        sc <- initial.sc
-        i <- 0
+        maxit <- 200
         eps <- 1e-20
-        ## magic number alert
-        err <- 1
-        while((i <- i+1) < max.it && err > eps) 
+        sc <- initial.sc
+
+        for(i in 1:maxit)
         {
             sc2 <- sqrt(sc^2 * mean(rho(u/sc, c1)) / kp)
-            err <- abs(sc2/sc - 1)
+            if(abs(sc2/sc - 1) <= eps)
+                break
             sc <- sc2
         }
-        sc
+        return(sc)
     }
 
     ##
-    ##  Do "k" IRWLS refining steps from "initial.mu, initial.sigma"
+    ##  Do "k" iterative reweighting refining steps from "initial.mu, initial.sigma"
     ##
     ##  If "initial.scale" is present, it's used, o/w the MAD is used
     ##
@@ -236,37 +317,34 @@ CovSest <- function(x,
     ##  conv = 1 means "stop when convergence is detected, or the
     ##                 maximum number of iterations is achieved"
     ##  kp and cc = tuning constants of the equation
-    ## 
-    re.s <- function(x, initial.mu, initial.sigma, initial.scale, k, conv, kp, cc) 
+    ##
+    re.s <- function(x, initial.mu, initial.sigma, initial.scale, k, conv, kp, cc)
     {
         n <- nrow(x)
         p <- ncol(x)
         rdis <- resdis(x, initial.mu, initial.sigma)
-        
-        if(missing(initial.scale)) 
+
+        if(missing(initial.scale))
         {
             initial.scale <- scale <- median(abs(rdis))/.6745
-        } else 
+        } else
         {
             scale <- initial.scale
         }
-        
+
         ## if conv == 1 then set the max no. of iterations to 50 magic number alert!!!
-        if (conv == 1) 
+        if(conv == 1)
             k <- 50
-        
+
         mu <- initial.mu
         sigma <- initial.sigma
-        lower.bound <- median(abs(rdis))/cc
-        for(i in 1:k) 
+        for(i in 1:k)
         {
             ## do one step of the iterations to solve for the scale
             scale.super.old <- scale
-        
-            ## lower.bound <- median(abs(rdis))/1.56
             scale <- sqrt(scale^2 * mean(rho(rdis/scale, cc)) / kp)
-        
-            ## now do one step of IRWLS with the "improved scale"
+
+            ## now do one step of reweighting with the "improved scale"
             weights <- scaledpsi(rdis/scale, cc)
             W <- weights %*% matrix(rep(1,p), ncol=p)
             xw <- x * W/mean(weights)
@@ -274,35 +352,34 @@ CovSest <- function(x,
             res <- x - matrix(rep(1,n),ncol=1) %*% mu.1
             sigma.1 <- t(res) %*% ((weights %*% matrix(rep(1,p), ncol=p)) * res)
             sigma.1 <- (det(sigma.1))^(-1/p) * sigma.1
-            
-            ## if(det(sigma.1) < 1e-7)     # singular
+
             if(.isSingular(sigma.1))
-            { 
+            {
                 mu.1 <- initial.mu
                 sigma.1 <- initial.sigma
                 scale <- initial.scale
                 break
             }
-            if(conv == 1) 
+            if(conv == 1)
             {
                 ## check for convergence
-                if(norm(mu - mu.1) / norm(mu) < 1e-20) 
+                if(norm(mu - mu.1) / norm(mu) < 1e-20)
                     break
                 ## magic number alert!!!
             }
-            
+
             rdis <- resdis(x,mu.1,sigma.1)
             mu <- mu.1
             sigma <- sigma.1
         }
-    
+
         rdis <- resdis(x,mu,sigma)
 
         ## get the residuals from the last beta
         return(list(mu.rw = mu.1, sigma.rw=sigma.1, scale.rw = scale))
     }
 
-################################################################################################    
+################################################################################################
     n <- nrow(x)
     p <- ncol(x)
 
@@ -311,55 +388,55 @@ CovSest <- function(x,
     best.scales <- rep(1e20, best.r)
     s.worst <- 1e20
     n.ref <- 1
-    
-    for(i in 1:nsamp) 
-    {   
-        ## Generate a p+1 subsample in general position and compute 
+
+    for(i in 1:nsamp)
+    {
+        ## Generate a p+1 subsample in general position and compute
         ##  its mu and sigma. If sigma is singular, generate another
         ##  subsample.
         ##  FIXME ---> the singularity check 'det(sigma) < 1e-7' can
-        ##      in some cases give TRUE, e.g. milk although 
-        ##      .isSingular(sigma) which uses qr(mat)$rank returns 
-        ##      FALSE. This can result in many retrials and thus 
+        ##      in some cases give TRUE, e.g. milk although
+        ##      .isSingular(sigma) which uses qr(mat)$rank returns
+        ##      FALSE. This can result in many retrials and thus
         ##      speeding down!
         ##
         singular <- TRUE
-        while(singular) 
+        while(singular)
         {
             indices <- sample(n, p+1)
             xs <- x[indices,]
             mu <- colMeans(xs)
-            sigma <- cov(xs) 
+            sigma <- cov(xs)
             ##singular <- det(sigma) < 1e-7
             singular <- .isSingular(sigma)
         }
-        sigma <- det(sigma)^(-1/p) * sigma    
+        sigma <- det(sigma)^(-1/p) * sigma
 
-        ## Perform k steps of IRLS on the elemental set
-        if(k > 0) 
-        { 
+        ## Perform k steps of iterative reweighting on the elemental set
+        if(k > 0)
+        {
             ## do the refining
             tmp <- re.s(x=x, initial.mu=mu, initial.sigma=sigma, k=k, conv=0, kp=kp, cc=cc)
             mu.rw <- tmp$mu.rw
             sigma.rw <- tmp$sigma.rw
             scale.rw <- tmp$scale.rw
             rdis.rw <- resdis(x, mu.rw, sigma.rw)
-        } else 
-        { 
+        } else
+        {
             ## k = 0 means "no refining"
             mu.rw <- mu
             sigma.rw <- sigma
             rdis.rw <- resdis(x, mu.rw, sigma.rw)
             scale.rw <- median(abs(rdis.rw))/.6745
         }
-        
-        if(i > 1) 
-        { 
+
+        if(i > 1)
+        {
             ## if this isn't the first iteration....
             ## check whether new mu/sigma belong to the top best results; if so keep
             ## mu and sigma with corresponding scale.
             scale.test <- loss.S(rdis.rw, s.worst, cc)
-            if(scale.test < kp) 
+            if(scale.test < kp)
             {
                 s.best <- scaleS(rdis.rw, kp, cc, scale.rw)
                 ind <- order(best.scales)[best.r]
@@ -369,8 +446,8 @@ CovSest <- function(x,
                 best.sigmas[(bm1+1):(bm1+p),] <- sigma.rw
                 s.worst <- max(best.scales)
             }
-        } else 
-        { 
+        } else
+        {
             ## if this is the first iteration, then this is the best solution anyway...
             best.scales[best.r] <- scaleS(rdis.rw, kp, cc, scale.rw)
             best.mus[best.r,] <- mu.rw
@@ -387,14 +464,14 @@ CovSest <- function(x,
         index <- (i-1)*p
         tmp <- re.s(x=x, initial.mu=best.mus[i,],
                     initial.sigma=best.sigmas[(index+1):(index+p),],
-                    initial.scale=best.scales[i], 
+                    initial.scale=best.scales[i],
                     k=0, conv=1, kp=kp, cc=cc)
-        if(tmp$scale.rw < super.best.scale) 
+        if(tmp$scale.rw < super.best.scale)
         {
             super.best.scale <- tmp$scale.rw
             super.best.mu <- tmp$mu.rw
             super.best.sigma <- tmp$sigma.rw
-        }   
+        }
     }
 
     super.best.sigma <- super.best.scale^2*super.best.sigma
@@ -411,10 +488,10 @@ CovSest <- function(x,
 ##  SURREAL algorithm using Tukey's biweight function
 ##      data    - the data: a matrix or a data.frame
 ##      nsamp   - number of random (p+1)-subsamples to be drawn, defaults to 600*p
-##      kp, c1  - tunning constants for the  S-estimator with Tukey's biweight 
+##      kp, c1  - tunning constants for the  S-estimator with Tukey's biweight
 ##                function given the breakdown point (bdp) and the dimension p
 ##
-..covSURREAL <- function(data, 
+..covSURREAL <- function(data,
                        nsamp,
                        kp,
                        c1,
@@ -422,7 +499,7 @@ CovSest <- function(x,
                        tol.inv)
 {
 
-## Compute the tunning constant c1 for the  S-estimator with 
+## Compute the tunning constant c1 for the  S-estimator with
 ##  Tukey's biweight function given the breakdown point (bdp)
 ##  and the dimension p
 vcTukey<-function(bdp, p)
@@ -440,14 +517,14 @@ vcTukey<-function(bdp, p)
         error <- abs(cold-cnew)
         contit <- contit+1
     }
-    if(contit == maxit) 
+    if(contit == maxit)
         warning(" Maximum iterations, the last values for c1 and kp are:")
     return (list(c1 = cnew, kp = kp))
 }
 
 ## Compute the constant kp (used in vcTukey)
 vkpTukey<-function(c0, p)
-{  
+{
    c2 <- c0^2
    intg1 <- (p/2)*pchisq(c2, (p+2))
    intg2 <- (p+2)*p/(2*c2)*pchisq(c2, (p+4))
@@ -481,7 +558,7 @@ scaleS <- function(dx, S0, c1, kp, tol)
             {
                 delta <- delta/2
                 mmin <- mmin+1
-            }   
+            }
         }
         test <- if(mmin == 10) 0 else (abs(rhoold)-abs(rhonew))/abs(rhonew)
         rhoold <- rhonew
@@ -511,9 +588,12 @@ psibiweight <- function(xx, c1)
     ## TEMPORARY for testing
     ##  ignore kp and c1
     v1 <- vcTukey(0.5, m)
+    if(trace)
+        cat("\nSURREAL..: bdp, p, c1, kp=", 0.5, m, v1$c1, v1$kp, "\n")
+
     c1 <- v1$c1
-    kp <- v1$kp    
-  
+    kp <- v1$kp
+
     ma <- m+1
     mb <- 1/m
     tol <- 1e-5
@@ -522,7 +602,7 @@ psibiweight <- function(xx, c1)
     laux <- 1
     for(l in 1:nsamp)
     {
-        ## generate a random subsample of size p+1 and compute its 
+        ## generate a random subsample of size p+1 and compute its
         ##  mean 'muJ' and covariance matrix 'covJ'
         singular <- TRUE
         while(singular)
@@ -535,10 +615,10 @@ psibiweight <- function(xx, c1)
 
         if(l > ceiling(nsamp*0.2))
         {
-            if(l == ceiling(nsamp*(0.5))) 
+            if(l == ceiling(nsamp*(0.5)))
                 laux <- 2
-            if(l == ceiling(nsamp*(0.8))) 
-                laux <- 4 
+            if(l == ceiling(nsamp*(0.8)))
+                laux <- 4
 
             epsilon <- runif(1)
             epsilon <- epsilon^laux
@@ -550,9 +630,9 @@ psibiweight <- function(xx, c1)
         md<-sqrt(mahalanobis(data, muJ, covJ, tol.inv=tol.inv))
         if(mean(rhobiweight(md/Stil, c1)) < kp)
         {
-            if(Stil < 5e10)  
+            if(Stil < 5e10)
                 Stil <- scaleS(md, Stil, c1, kp, tol)
-            else 
+            else
                 Stil <- scaleS(md, 0, c1, kp, tol)
             mutil <- muJ
             covtil <- covJ
@@ -576,14 +656,14 @@ psibiweight <- function(xx, c1)
                     mutil <- muJ
                     covtil <- covJ
                     control <- 1
-                    if(Stil < 5e10) 
+                    if(Stil < 5e10)
                         Stil<-scaleS(md, Stil, c1, kp, tol)
-                    else 
+                    else
                         Stil<-scaleS(md, 0, c1, kp, tol)
                 }else
                 {
                     muJ <- (muJ + mutil)/2
-                    covJ <- (covJ + covtil)/2 
+                    covJ <- (covJ + covtil)/2
                     covJ <- det(covJ) ^ -mb * covJ
                 }
             }
@@ -591,7 +671,7 @@ psibiweight <- function(xx, c1)
     }
 
     covtil <- Stil^2 * covtil
-    
+
     return (list(center = mutil,
                  cov = covtil,
                  crit = Stil,
@@ -599,7 +679,7 @@ psibiweight <- function(xx, c1)
                  method="S-estimates: SURREAL")
             )
 }
- 
+
 ##
 ## Compute the S-estimate of location and scatter using the  bisquare function
 ##
@@ -607,18 +687,18 @@ psibiweight <- function(xx, c1)
 ##          the rocke type estimates with the following diferences:
 ##          - the call to .iter.rocke() instead of iter.bic()
 ##          - the mahalanobis distance returned by iter.bic() is sqrt(mah)
-##          
+##
 ##  arp     - not used, i.e. used only in the Rocke type estimates
 ##  eps     - Iteration convergence criterion
 ##  maxiter - maximum number of iterations
 ##  t0, S0  - initial HBDM estimates of center and location
-##  nsamp, seed - if not provided T0 and S0, these will be used for CovMve() 
+##  nsamp, seed - if not provided T0 and S0, these will be used for CovMve()
 ##              to compute them
-##  initcontrol - if not provided t0 and S0, initcontrol will be used to compute 
+##  initcontrol - if not provided t0 and S0, initcontrol will be used to compute
 ##              them
 ##
-..covSBic <- function(x, arp, eps, maxiter, t0, S0, nsamp, seed, initcontrol){
-    
+..covSBic <- function(x, arp, eps, maxiter, t0, S0, nsamp, seed, initcontrol, trace=FALSE){
+
     dimn <- dimnames(x)
     dx <- dim(x)
     n <- dx[1]
@@ -631,8 +711,8 @@ psibiweight <- function(xx, c1)
     x <- scale(x, center = mu, scale=devin)
 
     ## If not provided initial estimates, compute them as MVE
-    ##  Take the raw estimates and standardise the covariance 
-    ##  matrix to determinant=1 
+    ##  Take the raw estimates and standardise the covariance
+    ##  matrix to determinant=1
     if(missing(t0) || missing(S0)){
         if(missing(initcontrol))
             init <- CovMve(x, nsamp=nsamp, seed=seed)
@@ -642,7 +722,7 @@ psibiweight <- function(xx, c1)
         if(cl == "CovMve" || cl == "CovMcd" || cl == "CovOgk"){
             t0 <- init@raw.center
             S0 <- init@raw.cov
-            
+
 ##            xinit <- cov.wt(x[init@best,])
 ##            t0 <- xinit$center
 ##            S0 <- xinit$cov
@@ -655,16 +735,16 @@ psibiweight <- function(xx, c1)
             S0 <- init@cov
         }
     }
-    
+
     ## initial solution
     center <- t0
     cov <- S0
-    
-    out <- .iter.bic(x, center, cov, maxiter, eps)
+
+    out <- .iter.bic(x, center, cov, maxiter, eps, trace=trace)
     center <- out$center
     cov <- out$cov
     mah <- out$mah
-    iter <- out$iter  
+    iter <- out$iter
 
     mah <- mah^2
     med0 <- median(mah)
@@ -697,8 +777,8 @@ psibiweight <- function(xx, c1)
 ##
 ##  see the notes to ..covSBic()
 ##
-..covSRocke <- function(x, arp, eps, maxiter, t0, S0, nsamp, seed, initcontrol){
-    
+..covSRocke <- function(x, arp, eps, maxiter, t0, S0, nsamp, seed, initcontrol, trace=FALSE){
+
     dimn <- dimnames(x)
     dx <- dim(x)
     n <- dx[1]
@@ -711,8 +791,8 @@ psibiweight <- function(xx, c1)
     x <- scale(x, center = mu, scale=devin)
 
     ## If not provided initial estimates, compute them as MVE
-    ##  Take the raw estimates and standardise the covariance 
-    ##  matrix to determinant=1 
+    ##  Take the raw estimates and standardise the covariance
+    ##  matrix to determinant=1
     if(missing(t0) || missing(S0)){
         if(missing(initcontrol))
             init <- CovMve(x, nsamp=nsamp, seed=seed)
@@ -722,7 +802,7 @@ psibiweight <- function(xx, c1)
         if(cl == "CovMve" || cl == "CovMcd" || cl == "CovOgk"){
             t0 <- init@raw.center
             S0 <- init@raw.cov
-            
+
 ##            xinit <- cov.wt(x[init@best,])
 ##            t0 <- xinit$center
 ##            S0 <- xinit$cov
@@ -735,16 +815,16 @@ psibiweight <- function(xx, c1)
             S0 <- init@cov
         }
     }
-    
+
     ## initial solution
     center <- t0
     cov <- S0
-    
-    out <- .iter.rocke(x, center, cov, maxiter, eps, alpha=arp)
+
+    out <- .iter.rocke(x, center, cov, maxiter, eps, alpha=arp, trace=trace)
     center <- out$center
     cov <- out$cov
     mah <- out$mah
-    iter <- out$iter  
+    iter <- out$iter
 
 #    mah <- mah^2
     med0 <- median(mah)
@@ -772,8 +852,8 @@ psibiweight <- function(xx, c1)
            )
 }
 
-.iter.bic <- function(x, center, cov, maxiter, eps)
-{ 
+.iter.bic <- function(x, center, cov, maxiter, eps, trace)
+{
 
     stepS <- function(x, b, cc, mah, n, p)
     {
@@ -781,7 +861,7 @@ psibiweight <- function(xx, c1)
         w <- w.bi(mah,cc)
         w.v <- matrix(w,n,p)
         tn <- colSums(w.v*x)/sum(w)
-        xc <- x-t(matrix(tn,p,n)) 
+        xc <- x-t(matrix(tn,p,n))
         vn <- t(xc)%*%(xc*w.v)
         eii <- eigen(vn, symmetric=TRUE, only.values = TRUE)
         eii <- eii$values
@@ -790,27 +870,27 @@ psibiweight <- function(xx, c1)
         condn <- bb/aa
         cn <- NULL
         fg <- TRUE
-        
+
         if(condn > 1.e-12)
         {
             fg <- FALSE
             cn <- (det(vn)^p2)*vn
         }
-    
+
         out1 <- list(tn, cn, fg)
         names(out1) <- c("center", "cov", "flag")
         return(out1)
     }
-    
+
     scale.full <- function(u, b, cc)
-    { 
-        #find the scale - full iterations 
+    {
+        #find the scale - full iterations
         max.it <- 200
         s0 <- median(abs(u))/.6745
         s.old <- s0
         it <- 0
         eps <- 1e-4
-        err <- eps + 1 
+        err <- eps + 1
         while(it < max.it && err > eps)
         {
             it <- it+1
@@ -820,7 +900,7 @@ psibiweight <- function(xx, c1)
         }
         return(s.old)
     }
-    
+
     rho.bi <- function(x, cc)
     {
         ## bisquared rho function
@@ -829,7 +909,7 @@ psibiweight <- function(xx, c1)
         ro1[!dif] <- cc^2/6
         return(ro1)
     }
-    
+
     psi.bi <- function(x, cc)
     {
         ## bisquared psi function
@@ -838,7 +918,7 @@ psibiweight <- function(xx, c1)
         psi1[!dif] <- 0
         return(psi1)
     }
-    
+
     w.bi <- function(x, cc)
     {
         ## bicuadratic w function
@@ -847,7 +927,7 @@ psibiweight <- function(xx, c1)
         w1[!dif] <- 0
         w1
     }
-    
+
     s.iter <- function(u, b, cc, s0)
     {
         # does one step for the scale
@@ -862,11 +942,14 @@ psibiweight <- function(xx, c1)
     ##  main routine for .iter.bic()
     cc <- 1.56
     b <- cc^2/12
-    
+
     dimx <- dim(x)
     n <- dimx[1]
     p <- dimx[2]
 
+    if(trace)
+        cat("\nbisquare.: bdp, p, c1, kp=", 0.5, p, cc, b, "\n")
+
     mah <- sqrt(mahalanobis(x, center, cov))
     s0 <- scale.full(mah, b, cc)
     mahst <- mah/s0
@@ -880,13 +963,13 @@ psibiweight <- function(xx, c1)
         it <- it + 1
         out <- stepS(x, b, cc, mahst, n, p)
         fg <- out$flag
-        if(!fg) 
+        if(!fg)
         {
             center <- out$center
-            cov <- out$cov 
+            cov <- out$cov
             mah <- sqrt(mahalanobis(x, center, cov))
             s <- scale.full(mah, b, cc)
-            mahst <- mah/s   
+            mahst <- mah/s
             u <- abs(s-s0)/s0
             s0 <- s
         }
@@ -896,8 +979,8 @@ psibiweight <- function(xx, c1)
 }
 
 
-.iter.rocke <- function(x, mu, v, maxit, tol, alpha)
-{ 
+.iter.rocke <- function(x, mu, v, maxit, tol, alpha, trace)
+{
 
 
 rho.rk2 <- function(t, p, alpha)
@@ -946,11 +1029,11 @@ disesca <- function(x, mu, v, alpha, delta)
     dis <- maha$dist
     fg <- maha$flag
     detv <- maha$det
-    
+
     sig <- NULL
     v1 <- NULL
     dis1 <- NULL
-    if(!fg) 
+    if(!fg)
     {
         dis1 <- dis*(detv^p2)
         v1 <- v*(detv^p3)
@@ -962,7 +1045,7 @@ disesca <- function(x, mu, v, alpha, delta)
 
 fespro <- function(sig, fp, fdis, falpha, fdelta)
 {
-    z <- fdis/sig 
+    z <- fdis/sig
     sfun <- mean(rho.rk2(z, fp, falpha)) - fdelta
     return(sfun)
 }
@@ -999,10 +1082,10 @@ gradien <- function(x, mu1, v1, mu0, v0, sig0, dis0, sig1, dis1, alpha)
     n <- dimx[1]
     p <- dimx[2]
     delta <- 0.5*(1-p/n)
-    decre <- 0.  
+    decre <- 0.
     mumin <- mu0
     vmin <- v0
-    dismin <- dis0  
+    dismin <- dis0
     sigmin <- sig0
     for(i in 1:10) {
         gamma <- i/10
@@ -1010,16 +1093,16 @@ gradien <- function(x, mu1, v1, mu0, v0, sig0, dis0, sig1, dis1, alpha)
         v <- (1-gamma)*v0+gamma*v1
         dis.res <- disesca(x,mu,v,alpha,delta)
         if(!dis.res$flag) {
-            sigma <- dis.res$sig  
+            sigma <- dis.res$sig
             decre <- (sig0-sigma)/sig0
             if(sigma < sigmin) {
                 mumin <- mu
                 vmin <- dis.res$v
-                sigmin <- dis.res$sig 
+                sigmin <- dis.res$sig
                 dismin <- dis.res$dis
             }
         }
-    }      
+    }
     ans <- list(mu=mumin, v=vmin, sig=sigmin, dis=dismin)
     return(ans)
 }
@@ -1041,7 +1124,7 @@ descen1 <- function(x,mu,v,alpha,sig0, dis0)
     xc <- x- (matrix(1,n,1)%*%mui)
     va <- t(xc)%*%(xc*w.v)
     disi.res <- disesca(x,mui,va,alpha,delta)
-    if(disi.res$flag) 
+    if(disi.res$flag)
     {
         disi.res$sig <- sig0+1
         disi.res$v <- va
@@ -1051,7 +1134,7 @@ descen1 <- function(x,mu,v,alpha,sig0, dis0)
     flagi <- disi.res$flag
     deti<-disi.res$det
     vi <- disi.res$v
-    
+
     ans <- list(mui=mui,vi=vi,sig=sigi,dis=disi,flag=flagi)
     return(ans)
 }
@@ -1066,12 +1149,12 @@ invfg <- function(x, tol=1.e-12)
     d <- y$d
     cond <- min(d)/max(d)
     fl <- FALSE
-    if(cond < tol) 
+    if(cond < tol)
         fl <- TRUE
-    
+
     det <- prod(d)
     invx <- NULL
-    if(!fl) 
+    if(!fl)
         invx <- v%*%diag(1/d)%*%t(u)
     out <- list(inv=invx, flag=fl, det=det)
     return(out)
@@ -1090,7 +1173,7 @@ mahalanobisfl <- function(x, center, cov, tol=1.e-12)
 
     out <- list(dist=dist, flag=fg, det=det1)
     return(out)
-} 
+}
 
 ##################################################################################
 
@@ -1105,18 +1188,18 @@ mahalanobisfl <- function(x, center, cov, tol=1.e-12)
     sigmin <- dis.res$sig
     dismin <- dis.res$dis
     it <- 1
-    decre <- tol+1 
-    while(decre > tol && it <= maxit) 
+    decre <- tol+1
+    while(decre > tol && it <= maxit)
     {
         ##  do reweighting
         des1 <- descen1(x, mumin, vmin, alpha, sigmin, dismin)
         mu1 <- des1$mui
         v1 <- des1$vi
         dis1 <- des1$dis
-        sig1 <- des1$sig    
+        sig1 <- des1$sig
         decre1 <- (sigmin-sig1)/sigmin
         if(decre1 <= tol) {
-            grad <- gradien(x,mu1,v1,mumin,vmin,sigmin, dismin, sig1,dis1,alpha)             
+            grad <- gradien(x,mu1,v1,mumin,vmin,sigmin, dismin, sig1,dis1,alpha)
             mu2 <- grad$mu
             v2 <- grad$v
             sig2 <- grad$sig
